/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package groovy.xml

import groovy.xml.DOMBuilder
import groovy.xml.dom.DOMCategory

/**
* Tests for the Groovy Xml user guide related to DOMCategory.
*
* @author Groovy Documentation Community
*/
class UserGuideDOMCategoryTest  extends GroovyTestCase {

    // tag::testXML[]
    static def CAR_RECORDS = '''
    <records>
      <car name='HSV Maloo' make='Holden' year='2006'>
        <country>Australia</country>
        <record type='speed'>Production Pickup Truck with speed of 271kph</record>
      </car>
      <car name='P50' make='Peel' year='1962'>
        <country>Isle of Man</country>
        <record type='size'>Smallest Street-Legal Car at 99cm wide and 59 kg in weight</record>
      </car>
      <car name='Royale' make='Bugatti' year='1931'>
        <country>France</country>
        <record type='price'>Most Valuable Car at $15 million</record>
      </car>
    </records>
  '''
  // end::testXML[]

  void testExample1() {
      // tag::testExample1[]
      def reader = new StringReader(CAR_RECORDS)
      def doc = DOMBuilder.parse(reader) //<1>
      def records = doc.documentElement

      use(DOMCategory) { // <2>
          assert records.car.size() == 3
      }
      // end::testExample1[]
  }

}
